/**************************************************************

   Program Title:  IMU64X0 Data Logger
   
   Program Name:  IMU64X0_Init.c	 Version: 1.00
   
   Author:  SS	  					  Date:  Feb 22, 2013
   
   Compiled with AVR Studio 4.19
   
   This file contains routines that implement the basic initialization 
   routines for the IMU6410 Monitor.

   Support for IMU6420 added.
   
   Copyright 2004-2013, SOC Robotics, Inc.  All rights reserved.
   
****************************************************************/
/*
 * Copyright 2004-2013, SOC Robotics, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Robotics, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SOC ROBOTICS, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-robotics.com
 */

/*
 * $Log: IMU64X0_Init.c,v $
 * Revision 1.00  2013/02/22 SS
 *    Support for ZigBee added
 * Revision 0.86  2006/02/07  SS
 *    2-wire interface, 8MHz clock, AVR Flash writer, XON/XOFF, 38,700baud
 * Revision 0.52  2005/06/04 18:36:42  SS
 * Preview release
 *
 */
// Include header files for GCC/IAR
#include "avr_compiler.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "IMU64X0_Defines.h"
#include "IMU64X0_Type.h"
#include "IMU64X0_Uart.h"
#include "IMU64X0_Extern.h"

//****************************************************************
//****************************************************************
// Initialization code
//****************************************************************
//****************************************************************

// Initialize UART0
void InitUART(unsigned char baudrate)
{
	unsigned char x;
	
	// Set the baud rate
	UBRR0H = 0x00;
	UBRR0L = baudrate; 
	// Enable UART receiver and transmitter
	UCSR0B = BIT(RXCIE0)|BIT(UDRIE0)|BIT(RXEN0)|BIT(TXEN0); 
    //  8 data bits, 1 stop bit, no parity
	UCSR0C = (3<<UCSZ00);
    //  8 data bits, 2 stop bit, no parity
//	UCSR0C = (3<<UCSZ00)|(1<<USBS0);

	x = 0; // flush receive buffer
	rxcnt = x; // rx buffer empty
	xoffsent = NO;
	sendxoff = NO;
	xonsent = NO;
	checkflag = NO;
	enablexonoff = YES; // turn serial driver xon/xoff flow control on
	echoinchars = NO;   // echo all received characters
	UART_RxTail = x;
	UART_RxHead = x;
	UART_TxTail = x;
	UART_TxHead = x;
	dataoverrun = NO;
	frameerror = NO;
	xoncnt = 0;
	xoffcnt = 0;
}

// Initialize UART1
void InitUART1(unsigned char baudrate)
{
	unsigned char x;
	
	// Set the baud rate
	UBRR1H = 0x00;
	UBRR1L = baudrate; 
	// Enable UART receiver and transmitter
	UCSR1B = BIT(RXCIE1)|BIT(UDRIE1)|BIT(RXEN1)|BIT(TXEN1); 
    //  8 data bits, 1 stop bit, no parity
	UCSR1C = (3<<UCSZ10);
    //  8 data bits, 2 stop bit, no parity
//	UCSR0C = (3<<UCSZ00)|(1<<USBS0);

	x = 0; // flush receive buffer
	rxcnt1 = x; // rx buffer empty
	xoffsent1 = NO;
	sendxoff1 = NO;
	xonsent1 = NO;
	checkflag1 = NO;
	enablexonoff1 = YES; // turn serial driver xon/xoff flow control on
	echoinchars1 = NO;   // echo all received characters
	UART_RxTail1 = x;
	UART_RxHead1 = x;
	UART_TxTail1 = x;
	UART_TxHead1 = x;
	dataoverrun1 = NO;
	frameerror1 = NO;
	xoncnt1 = 0;
	xoffcnt1 = 0;
}

//  Setup the Analog to Digital converter
//  TBD - convert to background interrupt process
void InitADC(void)
{
	DDRA = 0x00;  // All inputs
		   		  // Setup Analog TDB
}

//  Start the real time clock 
//  Clocked by external 32.768KHz watch crystal
//  Set prescalar to 0, set to interrupt using compare match 
//  set to 128 (4msec interrupt rate).  
void InitRealTimeClock(void)
{  
    // Software counters - approximately 4msec resolution
	rtccountdown1 = 0;  
	rtccountdown2 = 0;
	rtccountdown3 = 0;
	rtccountdown4 = 0;
	rtcclockcnt = 0;
	seconds = 0;
	minutes = 0;
	hours = 0;
	days = 0;
	month = 2;
	year = 2013;
	
 	TCNT2 = 0;
 	// Select external real time clock oscillator
	ASSR = (1<<AS2);
	// Set match interrupt to 4msec
	OCR2A = 131;
	// Reload counter on match interrupt - pre-scalar set to 0
	TCCR2A = (1<<WGM21);   
	TCCR2B = (1<<CS20);   
    // setup interrupts for real time clock
    TIMSK2 |= (1<<OCIE2A);
}

// Timer2 is called every 1msec - used to maintain four count down 
// software timers to control precise time delays in various routines
void Init_Timer0(void)
{
 	 mseccountdown1 = 0;
	 mseccountdown2 = 0;
	 mseccountdown3 = 0;
	 mseccountdown4 = 0;
	 flashcountdown = 0;
	 bmp085countdown = 0;
	 
	 TCNT0 = 0;
	 // Set mode 2 - reset counter on compare match, clk/64
	 TCCR0A = (1<<WGM01);
	 TCCR0B = (1<<WGM01)|(1<<CS01)|(1<<CS00);
	 
	 // Reset count after 1 msec (8MHz clock)
#ifdef  C8MCLOCK
	 OCR0A = 125;
#endif
	 
	 // Reset count after 1 msec (10MHz clock)
#ifdef  C10MCLOCK
	 OCR0A = 156;
#endif

	 // Reset count after 1 msec (11.0592MHz clock)
#ifdef  C11MCLOCK
	 OCR0A = 173;
#endif
	 	 
	 // Reset count after 1 msec (20.0MHz clock)
#ifdef  C20MCLOCK
	 OCR0A = 255;
#endif

	 // Enable compare match interrupt vector
	 TIMSK0 |= (1<<OCIE0A);
}

// Setup watch dog timer - TBD
void InitWatchdog(void)
{
}


//************************************************************
//  Initialize the AVR - Set Port conditions for IMU6410
//************************************************************
void Init_AVR(void)
{
	int i;

    // Setup serial flash constants and variables
	programmode = LFLASH;  // programming serial flash - default
	flashprogrammingactive = NO;
	flashpageloaded = NO;
	flashpageboundarytraversed = NO;
	flashdatainbuffer = NO;
	intelsuperaddress = 0;
	oldpage = 0;
	oldbyteoffset = 0;
	
    // Setup Port A
	InitADC();
	
	// Setup Port B
	//   PB0 - NC
	//   PB1 - ADXL345 CS
	//   PB2 - L3G4200 CS
	//   PB3 - Chip Select for uSD on IMU6410
	//   PB4 - SS - Chip select for uSD on IMU6420
//	DDRB = 0x16;
//	PORTB |= 0x16;  // If ADXL345 attached PB1 is chip select set high
	DDRB = 0x1E;
	PORTB = 0x1E;
	
#ifdef  IMU6410
	// Setup Port C for IMU6410
	//    PC0 is SCL - must be output
	//    PC1 is SDA
	//    PC2 XCLR BMP085
	//    PC3,4,5 Digital IO
	//    PC5  Green LED
	//    PC6,7 - 32.756KHz crystal
	//DDRC = 0x04;
	DDRC = 0x05;
	PORTC = 0x05;

	// Setup Port D
	//   IMU6410 Pin assignments:
	//     PD2 - Mag DRDY
	//     PD6 - Gyro INT2
	//     PD7 - Accel INT2
	//     PD5 - Red LED
	//     PD4 - Green LED
    DDRD = 0x30;

#else
	// Setup Port C for IMU6420
	//    PC0 is SCL - must be output
	//    PC1 is SDA
	//    PC2 XCLR BMP085
	//    PC3,4,5 Digital IO
	//    PC4  Red LED
	//    PC5  Green LED
	//    PC6,7 - 32.756KHz crystal
	//DDRC = 0x04;
	DDRC = 0x35;
	PORTC = 0x35;
	
	// Setup Port D
	//   IMU6410 Pin assignments:
	//     PD2 - RXD1
	//     PD3 - TXD1
	//     PD4 - PWM - OC1B
	//     PD5 - PWM - OC1A
	//     PD6 - PWM - OC2B
	//     PD7 - PWM - OC2A
    DDRD = 0x30;	
#endif

	//   Setup SPI - SPI default inactive in this release
//    Init_SPI_Master();
	
	//    Turn on real time clock on PC7,6 Timer/Counter 2
	InitRealTimeClock();
	
	// Setup Timer 0 to interrupt a 1msec rate - this interrupt
	// controls four software delay timers used by various 
	// routines
	Init_Timer0();
	
	// Turn LEDs off
	RED_LED(OFF);
	GREEN_LED(OFF);
	
	// Watchdog timer setup
//	InitWatchdog();
	
	// Initialize UART - Interrupt
	// Set the baudrate to 38,400 bps using an 8MHz clock
#ifdef C10MCLOCK
//	InitUART(BAUDRATE9600); 
//  InitUART(BAUDRATE19200);
    InitUART(BAUDRATE38400); 
//    InitUART(BAUDRATE57600); 
//    InitUART(BAUDRATE115200); 
#endif

// Default IMU6410 clock is 11.0592MHz
#ifdef C11MCLOCK
//	InitUART(BAUDRATE9600); 
//  InitUART(BAUDRATE19200);
//  InitUART(BAUDRATE38400); 
//  InitUART(BAUDRATE57600); 
    InitUART(BAUDRATE115200);
//	InitUART(BAUDRATE230400); 
#endif

#ifdef  ZIGBEEWIRELESS
    InitUART1(BAUDRATE38400);
#endif

#ifdef  ANTWIRELESS
    InitUART1(BAUDRATE57600);
#endif	

#ifdef C20MCLOCK
//	InitUART(BAUDRATE9600); 
//  InitUART(BAUDRATE19200);
    InitUART(BAUDRATE38400); 
//    InitUART(BAUDRATE57600); 
//    InitUART(BAUDRATE115200); 
#endif

	// ESC mode setting - if active ignore all commands until ESC received
	escmode = OFF;
	
    // IMU6410 System logging parameters
	defaultlograte = 10;
	defaultaccelerometerrange = G16MODE;
	defaultgyrorange = L250DPS;
	defaultmagnetometerrange = HMC130GA;
	logoutputmode = PROCESSED;
	minmax_mode = DISABLED;
	armpb0 = YES;
	armpb0 = NO;
	timedstartmode = NO;
	terminationmode = BUTTON;
	terminationmode = TIMEOUT;
	terminationmode = HALT;
	thresholdmode = INACTIVE;
	thresholdpercent = 10.0;
	for(i=0; i<12; i++) thresholdcheck[i] = 'y';
	timeoutperiod = 140;  // seconds
	data2log = 0x0f;  // Log accel, gyro, mag and baro
	dataformat = PROCESSED;

#ifdef IMU6410
	strcpy(deviceidtext,"IMU6410 V1.00 A");
	strcpy(deviceidtext2,"Right Elbow");
	strcpy(microsdname,"IMU6410.txt ");
#else
	strcpy(deviceidtext,"IMU6420 V1.00 A");
	strcpy(deviceidtext2,"Right Elbow");
	strcpy(microsdname,"IMU6420.txt ");
#endif

	fsynccnt = 0;
	//dataformat = PROCESSED;
//	deviceid = LEFTARM;
//	deviceid = RIGHTARM;
//	deviceid = UPPERSPINE;
//	deviceid = LOWERSPINE;
	deviceid = SLED;
	
	// DOF10W settings
	dof10wmode = ACTIVE;
	l3g4200dstate = 0;
	hmc5883state = 0;
	bmp085mode = INACTIVE;
	bmp085state = BMPTEMP;
	bmpskipcount = BMPSKIPVALUE;
	bmp085tempstate = BMPTEMPIDLE;
	bmp085pressurestate = BMPPRESSUREIDLE;
	printoutput = YES;

	// microSD state
	usdpresent = NO;
	storageselection = SERIALFLASH;
	fatmounted = NO;
	startloggingmode = NOTONPOWERUP;
	usdfilenamemode = UNIQUE;
	//usdfilenamemode = INCREMENTAL;
	logrestart = NO;
	logrestartcnt = 0;
	logrestartmax = 32000;
	usdfilenamenumber = 1;
	rtccnt = NO;

	// If DAD mode active don't output signon message and 
	// DAD commands immediately.  DAD mode is activated by
	// sending ESC d
	dadmode = INACTIVE;
	loggingmode = HALT;
	pagemode = NEW;

	// I2C Mode Settings
	i2cmode = ACTIVE;
	TWI_slaveAddress = TWI_SLAVEADDRESS;
	TWI_masterAddress = TWI_MASTERADDRESS;

	// See if signature bytes are loaded into EEPROM - if 
	// not there then write sig bytes and default preferences
	// otherwise load preferences
	if(!CheckEepromSignature())  {
	    printf("EEPROM contents not set\n\r");
	    WriteEepromSignature();
		WriteVersion();
		WriteCopyright();
	    SaveEepromPreferences();
	 	}
	else LoadEepromPreferences();

	// Enable TWI if active
	if(i2cmode==ACTIVE)  
	    TWI_Master_Initialize();
//	    TWI_Master_Initialize(TWI_masterAddress,TWI_GENERAL_CALL_ON,0x00);
					
}
