/**************************************************************

   Program Title:  IMU64X0 Data Logger
   
   Program Name:  IMU64X0_Motor.c		  Version: 1.00
   
   Author:  SS	  					  Date:  Feb 22, 2013
   
   Compiled with AVR Studio 4.19
   
   This file contains routines that implement basic motor control
   functions for a two axis wheelchair controller with integrated
   PWM controlled magnetic clutch.
   
   Copyright 2004-2013, SOC Robotics, Inc.  All rights reserved.
   
****************************************************************/
/*
 * Copyright 2004-2013, SOC Robotics, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Machines, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SOC ROBOTICS, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-robotics.com
 */

/*
 * $Log: IMU64X0_Init.c,v $
 * Revision 1.00  2013/02/22 SS
 *    Support for ZigBee added
 * Revision 0.52  2005/06/04 SS
 * Preview release
 *
 */
// Include header files for GCC/IAR
#include "avr_compiler.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "IMU64X0_Defines.h"
#include "IMU64X0_Type.h"
#include "IMU64X0_Uart.h"
#include "IMU64X0_Extern.h"

//****************************************************************
//****************************************************************
// Motor control functions
//****************************************************************
//****************************************************************

//Wheelchair Motor Control Features

//The following functions are supported:

#define  LEFTWHEEL  'l'
#define  RIGHTWHEEL 'r'
#define  FULLSTEP	'f'
#define  HALFSTEP	'h'
#define  QUARTERSTEP 'q'
#define  EIGTHSTEP	'8'
#define  SIXTEENTHSTEP '6'

// Set motor step mode: 1,2,4,8,16 microstep
void Set_Motor_StepMode(char stepmode,char wheel)
{
}

// Set motor torque
void Set_Motor_Torque(char torque,char wheel)
{
}

// Set motor decay
void Set_Motor_Decay(char decay,char wheel)
{
}

// Interrupt driven step engine
//   Number of steps, steprate in pulse per second, direction is 'c' CW or 'w' CCW and wheel -
//   r - right or l-left
void Step_Motor(long numsteps, unsigned int stepratepps,char direction,char wheel)
{
}

//  Step motor continuously until told to stop
void Step_Motor_Continuously(unsigned int stepratepps, char wheel)
{
}

//  Stop motor immediately
void Stop_Motor(char wheel)
{
}

// Turn mag drive 'o'-ON or 'f'-OFF for each wheel
void Mag_Drive(char state,char wheel)
{
}

//  Set the degree of magnetic clutch engagement - 0 - 255 with 256 being fully on
//  Frequency is compile time option - uses the timers PWM subsystem
void Set_Mag_PWM(unsigned char dutycycle, char wheel)
{
}


//  MM130 Functions
volatile unsigned char mm130motorcontrollerstate=0;


// Write float value to LED8 via TWI
void Write_Drive_MM130(char *outstrg)
{ 
	 TWI_Start_Write(MM130TWIADDRESS,outstrg,strlen(outstrg));
}

const char mm130mess1[] PROGMEM = "\n\rInput command to send to MM165\n\r";
const char mm130mess2[] PROGMEM = "Enter [1|2|3|4|5|6|7|8|l|h|e]: ";

//  Test function to control a MM130 via the I2C interface
/*void DriveMM130(void)
{
 	 char inchar;
	     
	 putfs(mm130mess1);
	 putfs(mm130mess2);
	 inchar = '1';
 	 while((inchar=getchar())!='e') {
	     putchar(inchar);  putCRLF();
		 if(inchar=='1') Write_Drive_MM130("dd10.15;3.45;");
		 else if(inchar=='2') Write_Drive_MM130("dd10.00;3.45;");
		 else if(inchar=='3') Write_Drive_MM130("dd20.15;3.45;");
		 else if(inchar=='4') Write_Drive_MM130("dd20.00;3.45;");
		 else if(inchar=='5') Write_Drive_MM130("dd80.15;3.45;");
		 else if(inchar=='6') Write_Drive_MM130("dd80.00;3.45;");
		 else if(inchar=='7') Write_Drive_MM130("dd60.15;3.45;");
		 else if(inchar=='8') Write_Drive_MM130("dd60.00;3.45;");
		 else if(inchar=='l') Write_Drive_MM130("q2");
		 else if(inchar=='h') Write_Drive_MM130("q1");
	 	 putfs(mm130mess2);
	     }
	 putCRLF();
}
*/

