/**************************************************************************
 **************************************************************************
 
  Program Title:  IMU64X0 Data Logger
  
  Program:  IMU64X0_TWI_Functions	  Version: 1.00
   
  Author:  SS	  					  Date:  Feb 22, 2013
   
  Compiled with AVR Studio 4.19.
  
  Copyright 2002-2013 SOC Robotics, Inc.
 
  TWI Functions for handling peripherals such as Motors, DRO's, joysticks
  etc.
		 
 ***************************************************************************	 
 ***************************************************************************/
// Include header files for GCC/IAR
#include "avr_compiler.h"
#include <avr/pgmspace.h>

#include <ctype.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
//#include <eeprom.h>
#include <stdio.h>

//#include "uart.h"
#include "IMU64X0_Uart.h"
#include "IMU64X0_Defines.h"
#include "IMU64X0_Type.h"
#include "IMU64X0_Extern.h" 

// ***********************************************************************
// ***********************************************************************
//     Start of Code
// ***********************************************************************
// ***********************************************************************


#define	   TWI_MTX_ADR_NACK  0x20

void Init_ADS1100_TWI(unsigned char twiaddress)
{
//unsigned char ADS1100writeAdd;
	//ADS1100writeAdd = (twiaddress << 1 | 0x00);
 	 messageBuf[0] = 0x0c;
	 TWI_Start_Write(ADS1100ADDRESS,messageBuf,0);
}

void Init_BMP085_TWI(unsigned char twiaddress)
{
 	 messageBuf[0] = 0x0c;
	 TWI_Start_Write(twiaddress,messageBuf,0);
}

/*
void Write_ADS1100_Register(unsigned char twiaddress,unsigned char outbyte)
{
    messageBuf[0] = outbyte;
    TWI_Start_Write(twiaddress,messageBuf,1);
}
*/
unsigned char Read_ADS1100_Register(unsigned char twiaddress,unsigned char iregister)
{	//unsigned char ADS1100readAdd;
	//ADS1100readAdd = (twiaddress << 1 | 0x01);
	TWI_Start_Read(ADS1100ADDRESS,iregister);
    TWI_Get_Data_From_Transceiver(messageBuf,1);
    return messageBuf[0];
}

// Configure SmartLCD display device via TWI to accept float
void  Init_SmartLCD_TWI(unsigned char twiaddress)
{
	 messageBuf[0] = ESC;
	 messageBuf[1] = 'c';
	 TWI_Start_Write(twiaddress,messageBuf,2);
}

// Start a measurement event in the Digital Compass
void  Init_DigitalCompass_TWI(void)
{
	 messageBuf[0] = 'W';  // Exit sleep state
	 TWI_Start_Write(DIGITALCOMPASSTWIADDRESS,messageBuf,1);
}

void Write_DigitalCompass_Register(unsigned char twiaddress,unsigned char outbyte)
{
    messageBuf[0] = outbyte;
    TWI_Start_Write(twiaddress,messageBuf,1);
}

unsigned char Read_DigitalCompass_Register(unsigned char twiaddress,unsigned char iregister)
{
	TWI_Start_Read(twiaddress,iregister);
    TWI_Get_Data_From_Transceiver(messageBuf,1);
    return messageBuf[0];
}

unsigned char Read_HMC5883_Register(unsigned char iregister)
{
	messageBuf[0] = iregister;
	TWI_Start_Write(HMC5883TWIADDRESS,messageBuf,1);
	TWI_Start_Read(HMC5883TWIADDRESS,1);
    TWI_Get_Data_From_Transceiver(messageBuf,1);
    return messageBuf[0];
}

void Write_HMC5883_Register(unsigned char iregister,unsigned char data)
{
	messageBuf[0] = iregister;
	messageBuf[1] = data;
	TWI_Start_Write(HMC5883TWIADDRESS,messageBuf,2);
}

void Init_HMC5883_TWI(void)
{
	messageBuf[0] = HMC5883_MODE_REG;
	messageBuf[1] = HMC5883_CONTINUOUS_M;  // Set continuous measurement mode
	TWI_Start_Write(HMC5883TWIADDRESS,messageBuf,2);
}

void Init_MS5611_TWI(void)
{
	messageBuf[0] = MS5611_RESET;
	TWI_Start_Write(MS5611TWIADDRESS,messageBuf,1);
}

void Init_BMA180_TWI()
{
 	 messageBuf[0] = 0x0c;
	 TWI_Start_Write(BMA180TWIADDRESS,messageBuf,0);
}

void Init_ITG3200_TWI()
{
 	 messageBuf[0] = 0x0c;
	 TWI_Start_Write(ITG3200TWIADDRESS,messageBuf,0);
}

unsigned char flipByte(unsigned char byteToFlip)
{		unsigned char flippedByte; 

		unsigned char flippedByteCopy; 
		flippedByte = 0x00;
		
		flippedByteCopy = byteToFlip;
		flippedByte |= (0b00001000 & (flippedByteCopy >> 1 ));
		flippedByte |= (0b00000100 & (flippedByteCopy >> 2 ));
		flippedByte |= (0b00000010 & (flippedByteCopy >> 2 ));
		flippedByte |= (0b00000001 & (flippedByteCopy >> 2 ));
		
		 
		flippedByteCopy = byteToFlip;
		flippedByte |= (0b00010000 & (flippedByteCopy << 1 ));
		flippedByte |= (0b00100000 & (flippedByteCopy << 2 ));
		flippedByte |= (0b01000000 & (flippedByteCopy << 2 ));
		flippedByte |= (0b10000000 & (flippedByteCopy << 2 ));
		 
		return flippedByte; 
}

unsigned int Read_ADS1100(void)
{		
	unsigned int returnValue;
	union {
	    unsigned char tempbuf[2];
		unsigned int outint;
		} u;
	
	u.outint = 0;
	TWI_Start_Read(ADS1100ADDRESS,3);
    TWI_Get_Data_From_Transceiver(messageBuf,3);
	u.tempbuf[1] = messageBuf[0];
	u.tempbuf[0] = messageBuf[1];
	returnValue = u.outint ;//+ 32768;
//	printf("\nmessageBuff0: %x\n\r", messageBuf[0]);
//	printf(" messageBuff1: %x\n\r", messageBuf[1]);
//	printf("messageBuff2: %x\n\r", messageBuf[2]);
    return returnValue;
}

unsigned char Read_BMP085_EEPROM(unsigned char eepromaddress)
{
	messageBuf[0] = eepromaddress;
    TWI_Start_Write(BMP085TWIADDRESS,messageBuf,1);
	messageBuf[0] = 0;
	TWI_Start_Read(BMP085TWIADDRESS,1);
    TWI_Get_Data_From_Transceiver(messageBuf,1);
	return messageBuf[0];
}

int Read_BMP085_EEPROM_Signed(unsigned char eepromaddress)
{
	union {
	    unsigned char tempbuf[2];
		int outint;
		} u;
	
	u.outint = 0;
	u.tempbuf[1] = Read_BMP085_EEPROM(eepromaddress++);
	//printf("\n\ru.tempbuf[1]=%x  ",u.tempbuf[1]);
	u.tempbuf[0] = Read_BMP085_EEPROM(eepromaddress);
	//printf("u.tempbuf[0]=%x\n\r",u.tempbuf[0]);
    return u.outint;  
}

unsigned int Read_BMP085_EEPROM_UnSigned(unsigned char eepromaddress)
{
	union {
	    unsigned char tempbuf[2];
		unsigned int outint;
		} u;
	
	u.outint = 0;
	u.tempbuf[1] = Read_BMP085_EEPROM(eepromaddress++);
	u.tempbuf[0] = Read_BMP085_EEPROM(eepromaddress);
    return u.outint;  
}

void Load_BMP085_Parameters(void)
{
 	 bmpac1=bmpac2=bmpac3=bmpb1=bmpb2=bmpmb=bmpmc=bmpmd=0;
	 bmpac4=bmpac5=bmpac6=0;

 	 // Read signed BMP085 Parameters
	 bmpac1 = Read_BMP085_EEPROM_Signed(0xaa);
	 bmpac2 = Read_BMP085_EEPROM_Signed(0xac);
	 bmpac3 = Read_BMP085_EEPROM_Signed(0xae);
	 bmpb1 = Read_BMP085_EEPROM_Signed(0xb6);
	 bmpb2 = Read_BMP085_EEPROM_Signed(0xb8);
	 bmpmb = Read_BMP085_EEPROM_Signed(0xba);
	 bmpmc = Read_BMP085_EEPROM_Signed(0xbc);
	 bmpmd = Read_BMP085_EEPROM_Signed(0xbe);
	 if(0) {
	     printf("\n\rBMP085 Calibration Parameters\n\r");
	     printf("bmpac1=%d",bmpac1);
	     printf(" %x\n\r",bmpac1);
	     printf("bmpac2=%d",bmpac2);
	     printf(" %x\n\r",bmpac2);
	     printf("bmpac3=%d",bmpac3);
	     printf(" %x\n\r",bmpac3);
	     printf("bmpb1=%d",bmpb1);
	     printf(" %x\n\r",bmpb1);
	     printf("bmpb2=%d",bmpb2);
	     printf(" %x\n\r",bmpb2);
	     printf("bmpmb=%d",bmpmb);
	     printf(" %x\n\r",bmpmb);
	     printf("bmpmc=%d",bmpmc);
	     printf(" %x\n\r",bmpmc);
	     printf("bmpmd=%d",bmpmd);
	     printf(" %x\n\r",bmpmd);
		 }
	 
	 // Read unsigned BMP085 parameters
	 bmpac4 = Read_BMP085_EEPROM_UnSigned(0xb0);
	 bmpac5 = Read_BMP085_EEPROM_UnSigned(0xb2);
	 bmpac6 = Read_BMP085_EEPROM_UnSigned(0xb4);
	 if(0) {
	     printf("bmpac4=%u",bmpac4);
	     printf(" %x\n\r",bmpac4);
	     printf("bmpac5=%u",bmpac5);
	     printf(" %x\n\r",bmpac5);
	     printf("bmpac6=%u",bmpac6);
	     printf(" %x\n\r",bmpac6);
		 }
}


void Start_Read_BMP085_Pressure(void)
{
    //printf("\n\rRead barometric pressure\n\r");
	messageBuf[0] = 0xf4;
	messageBuf[1] = 0xb4;
    TWI_Start_Write(BMP085TWIADDRESS,messageBuf,2);
}

unsigned int Finish_Read_BMP085_Pressure(void)
{
	union {
	    unsigned char tempbuf[2];
		unsigned int outint;
		} u;
	
	u.outint = 0;
	messageBuf[0] = 0xf6;
    TWI_Start_Write(BMP085TWIADDRESS,messageBuf,1);
	messageBuf[0] = 0;
	messageBuf[1] = 0;
	TWI_Start_Read(BMP085TWIADDRESS,2);
    TWI_Get_Data_From_Transceiver(messageBuf,2);
	u.tempbuf[1] = messageBuf[0];
	u.tempbuf[0] = messageBuf[1];
	//printf("\n\rmb[0]= %x ",messageBuf[0]);
	//printf("mb[1]= %x ",messageBuf[1]);
	//printf("%x\n\r",u.outint);
	
    return u.outint;
}

unsigned int Read_BMP085_Pressure(void)
{
	union {
	    unsigned char tempbuf[2];
		unsigned int outint;
		} u;
	
	u.outint = 0;
    //printf("\n\rRead barometric pressure\n\r");
	messageBuf[0] = 0xf4;
	messageBuf[1] = 0xb4;
    TWI_Start_Write(BMP085TWIADDRESS,messageBuf,2);
	Delaymsec(28);
	messageBuf[0] = 0xf6;
    TWI_Start_Write(BMP085TWIADDRESS,messageBuf,1);
	messageBuf[0] = 0;
	messageBuf[1] = 0;
	TWI_Start_Read(BMP085TWIADDRESS,2);
    TWI_Get_Data_From_Transceiver(messageBuf,2);
	u.tempbuf[1] = messageBuf[0];
	u.tempbuf[0] = messageBuf[1];
	//printf("\n\rpress mb[0]= %x ",messageBuf[0]);
	//printf("mb[1]= %x ",messageBuf[1]);
	//printf("%x\n\r",u.outint);
	
    return u.outint;
}

void Start_Read_BMP085_Temperature(void)
{
    //printf("\n\rRead temperature\n\r");
	messageBuf[0] = 0xf4;
	messageBuf[1] = 0x2e;
    TWI_Start_Write(BMP085TWIADDRESS,messageBuf,2);
}

unsigned int Finish_Read_BMP085_Temperature(void)
{
	union {
	    unsigned char tempbuf[2];
		unsigned int outint;
		} u;
	
	u.outint = 0;
	messageBuf[0] = 0xf6;
    TWI_Start_Write(BMP085TWIADDRESS,messageBuf,1);
	messageBuf[0] = 0;
	messageBuf[1] = 0;
	TWI_Start_Read(BMP085TWIADDRESS,2);
    TWI_Get_Data_From_Transceiver(messageBuf,2);
	u.tempbuf[1] = messageBuf[0];
	u.tempbuf[0] = messageBuf[1];
	//printf("mb[0]= %x\n\r",messageBuf[0]);
	//printf("mb[1]= %x\n\r",messageBuf[1]);
	
    return u.outint;
}

unsigned int Read_BMP085_Temperature(void)
{
	union {
	    unsigned char tempbuf[2];
		unsigned int outint;
		} u;
	
	u.outint = 0;
    //printf("\n\rRead temperature\n\r");
	messageBuf[0] = 0xf4;
	messageBuf[1] = 0x2e;
    TWI_Start_Write(BMP085TWIADDRESS,messageBuf,2);
	Delaymsec(7);
	messageBuf[0] = 0xf6;
    TWI_Start_Write(BMP085TWIADDRESS,messageBuf,1);
	messageBuf[0] = 0;
	messageBuf[1] = 0;
	TWI_Start_Read(BMP085TWIADDRESS,2);
    TWI_Get_Data_From_Transceiver(messageBuf,2);
	u.tempbuf[1] = messageBuf[0];
	u.tempbuf[0] = messageBuf[1];
	//printf("temp mb[0]= %x\n\r",messageBuf[0]);
	//printf("mb[1]= %x\n\r",messageBuf[1]);
	
    return u.outint;
}

unsigned long Read_MS5611_UnSigned(unsigned char address)
{
	union {
	    unsigned char tempbuf[4];
		unsigned long outlong;
		} u;
	
	messageBuf[0] = address;
    TWI_Start_Write(MS5611TWIADDRESS,messageBuf,1);
	Delaymsec(10);
	// read ADC
	messageBuf[0] = MS5611_ADC;
    TWI_Start_Write(MS5611TWIADDRESS,messageBuf,1);
	messageBuf[0] = 0;
	messageBuf[1] = 0;
	messageBuf[2] = 0;
	messageBuf[3] = 0;
	TWI_Start_Read(MS5611TWIADDRESS,3);
    TWI_Get_Data_From_Transceiver(messageBuf,3);
	u.outlong = 0;
	u.tempbuf[3] = messageBuf[0];
	u.tempbuf[2] = messageBuf[1];
	u.tempbuf[1] = messageBuf[2];
	printf("\n\r%x %x %x\n\r",messageBuf[0],messageBuf[1],messageBuf[2]);
	u.tempbuf[0] = 0;
    return u.outlong;  
}


unsigned int Read_MS5611_EEPROM_UnSigned(unsigned char eepromaddress)
{
	union {
	    unsigned char tempbuf[2];
		unsigned int outint;
		} u;
	
	messageBuf[0] = eepromaddress;
    TWI_Start_Write(MS5611TWIADDRESS,messageBuf,1);
	messageBuf[0] = 0;
	messageBuf[1] = 0;
	TWI_Start_Read(MS5611TWIADDRESS,2);
    TWI_Get_Data_From_Transceiver(messageBuf,2);
	u.outint = 0;
	u.tempbuf[1] = messageBuf[0];
	u.tempbuf[0] = messageBuf[1];
    return u.outint;  
}

void Load_MS5611_Parameters(void)
{
	unsigned char ms5611_c0;

	ms5611_c1 = ms5611_c2 = ms5611_c3 = ms5611_c4 = ms5611_c5 = 0;
	ms5611_c6 = ms5611_c7 = ms5611_c8 = 0;


		 // Read signed BMP085 Parameters
	 ms5611_c0 = Read_MS5611_EEPROM_UnSigned(0xA0);
	 ms5611_c1 = Read_MS5611_EEPROM_UnSigned(0xA2);
	 ms5611_c2 = Read_MS5611_EEPROM_UnSigned(0xA4);
	 ms5611_c3 = Read_MS5611_EEPROM_UnSigned(0xA6);
	 ms5611_c4 = Read_MS5611_EEPROM_UnSigned(0xA8);
	 ms5611_c5 = Read_MS5611_EEPROM_UnSigned(0xAA);
	 ms5611_c6 = Read_MS5611_EEPROM_UnSigned(0xAC);
	 ms5611_c7 = Read_MS5611_EEPROM_UnSigned(0xAE);

	 if(0) {
	     printf("\n\rMS5611 Calibration Parameters\n\r");
	     printf("ms5611_c1=%d",ms5611_c1);
	     printf(" %x\n\r",ms5611_c1);
	     printf("ms5611_c2=%d",ms5611_c2);
	     printf(" %x\n\r",ms5611_c2);
	     printf("ms5611_c3=%d",ms5611_c3);
	     printf(" %x\n\r",ms5611_c3);
	     printf("ms5611_c4=%d",ms5611_c4);
	     printf(" %x\n\r",ms5611_c4);
	     printf("ms5611_c5=%d",ms5611_c5);
	     printf(" %x\n\r",ms5611_c5);
	     printf("ms5611_c6=%d",ms5611_c6);
	     printf(" %x\n\r",ms5611_c6);
	     printf("ms5611_c7=%d",ms5611_c7);
	     printf(" %x\n\r",ms5611_c7);
		 }
}

unsigned int Read_DigitalCompass_Heading(void)
{
	union {
	    unsigned char tempbuf[2];
		unsigned int outint;
		} u;
	
	u.outint = 0;
	Write_DigitalCompass_Register(DIGITALCOMPASSTWIADDRESS,'A');
    //printf("\n\rRead digital compass\n\r");
	TWI_Start_Read(DIGITALCOMPASSTWIADDRESS,3);
    TWI_Get_Data_From_Transceiver(messageBuf,2);
	u.tempbuf[1] = messageBuf[0];
	u.tempbuf[0] = messageBuf[1];
	//printf("messagebug[0]=%x\n\r",messageBuf[0]);
	//printf("messagebug[1]=%x\n\r",messageBuf[1]);
	
    return u.outint;
}

// Write float value to LED8 via TWI
void Write_Float_SmartLCD(unsigned char twiaddress,unsigned char axisid,float fvalue)
{
 	 void *value;
	 
	 messageBuf[0] = 'm';
	 messageBuf[1] = axisid;
	 value = &messageBuf[2];
	 *(float *)value = fvalue;
	 TWI_Start_Write(twiaddress,messageBuf,6);
}

unsigned char Read_SmartLCD_Switches(char twiaddress)
{
	TWI_Start_Read(twiaddress,1);
    TWI_Get_Data_From_Transceiver(messageBuf,1);
    return messageBuf[0];
}

int Read_SmartLCD_Rotaryvalue(char twiaddress)
{
	union {
	    unsigned char tempbuf[2];
		unsigned long outint;
		} rotaryvalue;

	TWI_Start_Read(twiaddress,3);
    TWI_Get_Data_From_Transceiver(messageBuf,3);
	rotaryvalue.tempbuf[0] = messageBuf[2];
	rotaryvalue.tempbuf[1] = messageBuf[1];
    return rotaryvalue.outint;
}

void putcharSmartLCD(char inchar)
{	 
	 messageBuf[0] = inchar;
	 TWI_Start_Write(SMARTLCDTWIADDRESS,messageBuf,1);
}

void putsSmartLCD(char *strg)
{
 	unsigned char i;
	
 	for(i=0; i<strlen(strg); i++) {
	    putcharSmartLCD(strg[i]);
		Delaymsec(5);
		}
}

void Display_SmartLCD(void)
{	
	float xpos=0,ypos=0,zpos=0,apos=0;

	//xpos = ((float)xoldpos)*xresolution;
	//if(xdirpolarity==NEGATIVE)  xpos = -xpos;
	Write_Float_SmartLCD(SMARTLCDTWIADDRESS,'x',xpos);
	Delaymsec(4);
	
	//ypos = ((float)yoldpos)*yresolution;
	//if(ydirpolarity==NEGATIVE)  ypos = -ypos;
	Write_Float_SmartLCD(SMARTLCDTWIADDRESS,'y',ypos);
	Delaymsec(4);

	//zpos = ((float)zoldpos)*zresolution;
	//if(zdirpolarity==NEGATIVE)  zpos = -zpos;
	Write_Float_SmartLCD(SMARTLCDTWIADDRESS,'z',zpos);
	Delaymsec(4);
					  
	//apos = ((float)holdpos)*aresolution;
	//if(adirpolarity==NEGATIVE)  apos = -apos;
	Write_Float_SmartLCD(SMARTLCDTWIADDRESS,'a',apos);
	//delaymsec(5);
}

void ClearSmartLCD(void)
{
 	 Init_SmartLCD_TWI(SMARTLCDTWIADDRESS);
	 Delaymsec(10);
}

void Enumerate_SmartLCD(void)
{
 	 smartlcdstate = 0;
	 
	 Init_SmartLCD_TWI(SMARTLCDTWIADDRESS);
     if(TWI_Get_State_Info()!=TWI_MTX_ADR_NACK) {
	 	 smartlcdstate |= 0x01;
	 	 Delaymsec(50);
	 	 //Display_SmartLCD();
		 }
}

void Enumerate_BMP085(void)
{
	 BMPstate = bmpac1 = 0;

	 bmpac1 = Read_BMP085_EEPROM_Signed(0xaa);
	 //printf("bmpac1 = %d\n\r",bmpac1);
     if(bmpac1!=0) {
	 	 BMPstate |= 0x01;
	 	 Delaymsec(7);
		 }
}

void Enumerate_HMC5883(void)
{
 	 hmc5883state = 0;
	 
	 Init_HMC5883_TWI();
     if(TWI_Get_State_Info()!=TWI_MTX_ADR_NACK) {
	 	 hmc5883state |= 0x01;
	 	 Delaymsec(7);
		 }
}

void Enumerate_MS5611(void)
{
 	 ms5611state = 0;
	 
	 Init_MS5611_TWI();
     if(TWI_Get_State_Info()!=TWI_MTX_ADR_NACK) {
	 	 ms5611state |= 0x01;
	 	 Delaymsec(7);
		 }
}

void Enumerate_BMA180(void)
{
 	 bma180state = 0;
	 
	 Init_BMA180_TWI();
     if(TWI_Get_State_Info()!=TWI_MTX_ADR_NACK) {
	 	 bma180state |= 0x01;
		 //Read_DigitalCompass_Heading();  // read first wrong value
	 	 Delaymsec(7);
		 }
}

void Enumerate_ITG3200(void)
{
 	 itg3200state = 0;
	 
	 Init_ITG3200_TWI();
     if(TWI_Get_State_Info()!=TWI_MTX_ADR_NACK) {
	 	 itg3200state |= 0x01;
		 //Read_DigitalCompass_Heading();  // read first wrong value
	 	 Delaymsec(7);
		 }
}

const char enummes1[] PROGMEM = "\n\rEnumerate TWI Devices\n\r";
const char enummes8[] PROGMEM = " SmartLCD not found,";
const char enummes9[] PROGMEM = " SmartLCD found,";
const char enummes14[] PROGMEM = " BMP085 not found,";
const char enummes14b[] PROGMEM = " BMP180 not found,";
const char enummes15[] PROGMEM = " BMP085 found,";
const char enummes15b[] PROGMEM = " BMP180 found,";
const char enummes18[] PROGMEM = " HMC5883 not found\n\r";
const char enummes19[] PROGMEM = " HMC5883 found\n\r";
const char enummes20[] PROGMEM = " BMA180 not found, ";
const char enummes21[] PROGMEM = " BMA180 found, ";
const char enummes22[] PROGMEM = " ITG3200 not found\n\r";
const char enummes23[] PROGMEM = " ITG3200 found\n\r";

void Enumerate_TWI_Devices(void)
{	
	 putfs(enummes1);	
	 
	 // Enumerate BMP085 Sensor 1
	 Enumerate_BMP085();
	 if(BMPstate==0) {
#ifdef  IMU6410
	     putfs(enummes14);
#else
	     putfs(enummes14b);
#endif
	 }
	 else {
#ifdef  IMU6410
	    putfs(enummes15);
#else
	    putfs(enummes15b);
#endif
		}
	 
	 // Enumerate SmartLCD
	 Enumerate_SmartLCD();
	 if(smartlcdstate==0) putfs(enummes8);
	 else putfs(enummes9);
	 
	 Enumerate_HMC5883();
	 if(hmc5883state==0) putfs(enummes18);
	 else putfs(enummes19);
}

void Enumerate_Other_TWI_Devices()
{
	Enumerate_BMA180();
	if(bma180state==0) putfs(enummes20);
	else putfs(enummes21);

	Enumerate_ITG3200();
	if(itg3200state==0) putfs(enummes22);
	else putfs(enummes23);
}

const char TWImess1[] PROGMEM = "\n\rTWI Interactive Command Mode\n\r";
const char TWImess2[] PROGMEM = "Enter twi address (hex-two chars) followed by ASCII commands\n\r";
const char TWImess3[] PROGMEM = "  Example:  14xtx    Enter 'e' to exit\n\r";

// Send commands on I2C bus given twi address and an ASCII string
//  TBD - get data command
void Execute_TWI_Command(void)
{ 
     unsigned char twiaddress,iptr;
	  
	 putfs(TWImess1);
	 putfs(TWImess2);
	 putfs(TWImess3);
	 putchar(':');
 	 while((instrg[0]=tolower(getcharr()))!='e') {
	     instrg[1] = getcharr();
		 instrg[2] = 0;
		 sscanf(instrg,"%x",&twiaddress);
		 iptr = 0;
	     while((messageBuf[iptr++]=getcharr())!=CR) {
		     if(iptr==20) break;
			 }
		 messageBuf[iptr] = 0;
		 
		 printf("\n\rtwi=%x %s",twiaddress,messageBuf);
		 
		 TWI_Start_Write(twiaddress,messageBuf,iptr);

	     putCRLF(); putchar(':');
	     }
	 putCRLF();
}

